/*
 * cbcp - Call Back Configuration Protocol.
 *
 * Copyright (c) 1995 Pedro Roque Marques.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names of the authors of this software must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Pedro Roque Marques
 *     <pedro_m@yahoo.com>"
 *
 * THE AUTHORS OF THIS SOFTWARE DISCLAIM ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define RCSID	"$Id: cbcp.c,v 1.17 2006/05/22 00:04:07 paulus Exp $"

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>

#include "pppd.h"
#include "cbcp.h"
#include "fsm.h"
#include "lcp.h"
#include "ipcp.h"
#include "pathnames.h"
#include <sys/stat.h>


/*
 * Options.
 */
/*
int setcbcp __P((char **));

static option_t cbcp_option_list[] = {
    { "callback", o_special, (void *)setcbcp,
      "Ask for callback", OPT_PRIO | OPT_A2STRVAL, &cbcp[0].us_number },
    { NULL }
};
*/

/*
 * Protocol entry points.
 */
static void cbcp_init      __P((int unit));
static void cbcp_open      __P((int unit));
static void cbcp_lowerup   __P((int unit));
static void cbcp_input     __P((int unit, u_char *pkt, int len));
static void cbcp_protrej   __P((int unit));
static int  cbcp_printpkt  __P((u_char *pkt, int len,
				void (*printer) __P((void *, char *, ...)),
				void *arg));

struct protent cbcp_protent = {
    PPP_CBCP,
    cbcp_init,
    cbcp_input,
    cbcp_protrej,
    cbcp_lowerup,
    NULL,
    cbcp_open,
    NULL,
    cbcp_printpkt,
    NULL,
    0,
    "CBCP",
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

cbcp_state cbcp[NUM_PPP];	

/* internal prototypes */

static void cbcp_sendreq __P((void *arg));
static void cbcp_recvreq __P((cbcp_state *us, u_char *pckt, int len));
static void cbcp_sendresp __P((cbcp_state *us));
static void cbcp_recvresp __P((cbcp_state *us, char *pckt, int len));
static void cbcp_sendack __P((void *));
static void cbcp_recvack __P((cbcp_state *us, u_char *pckt, int len));
static void cbcp_send __P((cbcp_state *us, int code, u_char *buf, int len));
static void cbcp_make_options __P((int unit));
static int  cbcp_check_user __P((char *user, char *mask));
static void cbcp_start_callback __P((cbcp_state *us));
static void cbcp_up __P((cbcp_state *us));

cbcp_state *stop_iface = NULL;

void (*cbcp_init_hook) __P((cbcp_state *)) = NULL;
 

/* init state */
static void
cbcp_init(iface)
    int iface;
{
    cbcp_state *us;

    us = &cbcp[iface];
    memset(us, 0, sizeof(cbcp_state));
    us->us_unit = iface;
}

/* lower layer is up */
static void
cbcp_lowerup(iface)
    int iface;
{
    cbcp_state *us = &cbcp[iface];

    dbglog("cbcp_lowerup");
    dbglog("want: %d", us->us_type);
}

static void
cbcp_open(unit)
    int unit;
{
    dbglog("cbcp_open");
    if (lcp_hisoptions[unit].neg_cbcp)
    	cbcp_make_options(unit);
}

/* process an incomming packet */
static void
cbcp_input(unit, inpacket, pktlen)
    int unit;
    u_char *inpacket;
    int pktlen;
{
    u_char *inp;
    u_char code, id;
    u_short len;

    cbcp_state *us = &cbcp[unit];
    lcp_options *go  = &lcp_gotoptions[unit];
    lcp_options *his = &lcp_hisoptions[unit];

    inp = inpacket;

    if (pktlen < CBCP_MINLEN) {
	if (debug)
	    dbglog("CBCP packet is too small");
	return;
    }

    GETCHAR(code, inp);
    GETCHAR(id, inp);
    GETSHORT(len, inp);

    if (len > pktlen || len < CBCP_MINLEN) {
	if (debug)
	    dbglog("CBCP packet: invalid length %d", len);
        return;
    }

    len -= CBCP_MINLEN;
 
    switch(code) {
    case CBCP_REQ:
	if ( !go->neg_cbcp )
	{
	    dbglog("CBCP received CBCP_REQ, but CBCP running in server mode!");
	    return;
	}
        us->us_id = id;
	cbcp_recvreq(us, inp, len);
	break;

    case CBCP_RESP:
	if ( !his->neg_cbcp )
	{
	    dbglog("CBCP received CBCP_RESP, but CBCP running in client mode!");
	    return;
	}
	if (id != us->us_id)
	    dbglog("id doesn't match: expected %d recv %d",
		   us->us_id, id);

	cbcp_recvresp(us, inp, len);
	break;

    case CBCP_ACK:
	if ( !go->neg_cbcp )
	{
   	    dbglog("CBCP received CBCP_ACK, but CBCP running in server mode!");
	    return;
	}
	if (debug && id != us->us_id)
	    dbglog("id doesn't match: expected %d recv %d",
		   us->us_id, id);

	cbcp_recvack(us, inp, len);
	break;

    default:
	break;
    }
}

/* protocol was rejected by foe */
void cbcp_protrej(int iface)
{
}

char *cbcp_codenames[] = {
    "Request", "Response", "Ack"
};

char *cbcp_optionnames[] = {
    "NoCallback",
    "UserDefined",
    "AdminDefined",
    "List"
};

/* pretty print a packet */
static int
cbcp_printpkt(p, plen, printer, arg)
    u_char *p;
    int plen;
    void (*printer) __P((void *, char *, ...));
    void *arg;
{
    int code, opt, id, len, olen, delay;
    u_char *pstart;

    if (plen < HEADERLEN)
	return 0;
    pstart = p;
    GETCHAR(code, p);
    GETCHAR(id, p);
    GETSHORT(len, p);
    if (len < HEADERLEN || len > plen)
	return 0;

    if (code >= 1 && code <= sizeof(cbcp_codenames) / sizeof(char *))
	printer(arg, " %s", cbcp_codenames[code-1]);
    else
	printer(arg, " code=0x%x", code); 

    printer(arg, " id=0x%x", id);
    len -= HEADERLEN;

    switch (code) {
    case CBCP_REQ:
    case CBCP_RESP:
    case CBCP_ACK:
        while(len >= 2) {
	    GETCHAR(opt, p);
	    GETCHAR(olen, p);

	    if (olen < 2 || olen > len) {
	        break;
	    }

	    printer(arg, " <");
	    len -= olen;

	    if (opt >= 1 && opt <= sizeof(cbcp_optionnames) / sizeof(char *))
	    	printer(arg, " %s", cbcp_optionnames[opt-1]);
	    else
	        printer(arg, " option=0x%x", opt); 

	    if (olen > 2) {
	        GETCHAR(delay, p);
		printer(arg, " delay = %d", delay);
	    }

	    if (olen > 4) {
	        int addrt;
		char str[256];

		GETCHAR(addrt, p);
		memcpy(str, p, olen - 4);
		str[olen - 4] = 0;
		printer(arg, " number = %s", str);
	    }
	    printer(arg, ">");
	}
	break;

    default:
	break;
    }

    for (; len > 0; --len) {
	GETCHAR(code, p);
	printer(arg, " %.2x", code);
    }

    return p - pstart;
}

/* received CBCP request */
static void
cbcp_recvreq(us, pckt, pcktlen)
    cbcp_state *us;
    u_char *pckt;
    int pcktlen;
{
    u_char type, opt_len, addr_type;
    char address[256];
    int len = pcktlen;

    address[0] = 0;

    while (len >= 2) {
        dbglog("length: %d", len);

	GETCHAR(type, pckt);
	GETCHAR(opt_len, pckt);
	if (opt_len < 2 || opt_len > len)
	    break;

	us->us_delay = 0;
	if (opt_len > 2)
	    GETCHAR(us->us_delay, pckt);

	us->us_allowed |= (1 << type);

	switch(type) {
	case CB_CONF_NO:
	    dbglog("no callback allowed");
	    break;

	case CB_CONF_USER:
	    dbglog("user callback allowed");
	    if (opt_len > 4) {
	        GETCHAR(addr_type, pckt);
		memcpy(address, pckt, opt_len - 4);
		address[opt_len - 4] = 0;
		if (address[0])
		    dbglog("address: %s", address);
	    }
	    break;

	case CB_CONF_ADMIN:
	    dbglog("user admin defined allowed");
	    break;

	case CB_CONF_LIST:
	    break;
	}
	len -= opt_len;
    }
    if (len != 0) {
	if (debug)
	    dbglog("cbcp_recvreq: malformed packet (%d bytes left)", len);
	return;
    }

    cbcp_sendresp(us);
}

static void
cbcp_sendresp(us)
    cbcp_state *us;
{
    u_char cb_allowed;
    u_char buf[256];
    u_char *bufp = buf;
    int len = 0;
    int slen;

    cb_allowed = us->us_allowed;
    dbglog("cbcp_resp cb_type=%d", cb_allowed);

#if 0
    if (!cb_type)
        lcp_down(us->us_unit);
#endif

    if (cb_allowed & ( 1 << CB_CONF_USER ) ) {
	us->us_type = 1 << CB_CONF_USER;
	dbglog("cbcp_resp CONF_USER");
	slen = strlen(us->us_number);
	if (slen > 250) {
	    warn("callback number truncated to 250 characters");
	    slen = 250;
	}
	PUTCHAR(CB_CONF_USER, bufp);
	len = 3 + 1 + slen + 1;
	PUTCHAR(len, bufp);
	PUTCHAR(us->us_delay, bufp); /* delay */
	PUTCHAR(1, bufp);
	BCOPY(us->us_number, bufp, slen + 1);
	cbcp_send(us, CBCP_RESP, buf, len);
	return;
    }

    if (cb_allowed & ( 1 << CB_CONF_ADMIN ) ) {
	us->us_type = 1 << CB_CONF_ADMIN;
	dbglog("cbcp_resp CONF_ADMIN");
        PUTCHAR(CB_CONF_ADMIN, bufp);
	len = 3;
	PUTCHAR(len, bufp);
	PUTCHAR(us->us_delay, bufp); /* delay */
	cbcp_send(us, CBCP_RESP, buf, len);
	return;
    }

    if (cb_allowed & ( 1 << CB_CONF_NO ) ) {
	us->us_type = 1 << CB_CONF_NO;
        dbglog("cbcp_resp CONF_NO");
	PUTCHAR(CB_CONF_NO, bufp);
	len = 2;
	PUTCHAR(len , bufp);
	cbcp_send(us, CBCP_RESP, buf, len);
//	start_networks(us->us_unit);
	return;
    }
}

static void
cbcp_send(us, code, buf, len)
    cbcp_state *us;
    int code;
    u_char *buf;
    int len;
{
    u_char *outp;
    int outlen;

    outp = outpacket_buf;

    outlen = 4 + len;
    
    MAKEHEADER(outp, PPP_CBCP);

    PUTCHAR(code, outp);
    PUTCHAR(us->us_id, outp);
    PUTSHORT(outlen, outp);
    
    if (len)
        BCOPY(buf, outp, len);

    output(us->us_unit, outpacket_buf, outlen + PPP_HDRLEN);
}

static void
cbcp_recvack(us, pckt, len)
    cbcp_state *us;
    u_char *pckt;
    int len;
{
    u_char type, addr_type;
    int opt_len;
    char address[256];

    stop_iface = us;

    if (len >= 2) {
        GETCHAR(type, pckt);
	GETCHAR(opt_len, pckt);
	if (opt_len >= 2 && opt_len <= len) {
     
	    if (opt_len > 2)
		GETCHAR(us->us_delay, pckt);

	    if (opt_len > 4) {
		GETCHAR(addr_type, pckt);
		memcpy(address, pckt, opt_len - 4);
		address[opt_len - 4] = 0;
		if (address[0])
		    dbglog("peer will call: %s", address);
	    }
	    if (type != CB_CONF_NO)
	    {
		callback_in_progress = us->us_unit + 1;
		callback_in_progress |=  CBCP_CLIENT;
		cbcp_up(us);
	    }
	    else
		network_phase(us->us_unit);

	} else if (debug)
	    dbglog("cbcp_recvack: malformed packet");
    }
}

/* Make options
   if auth req, options from callback-users file, else use CBCP_CONF_USER */
    static void
cbcp_make_options (unit)
    int	unit;
{
    cbcp_state *us = &cbcp[unit];
    FILE *userfile;
    struct stat sbuf;
    int best_fit, got_fit, newline;
    char  uname[ 256 ], option[ 256 ];    

    us->us_id = 1;
    us->us_count = 0;
    us->us_delay = 5; /* Default delay 5 seconds */
    if ( *peer_authname ) { /* Username available */
	userfile = fopen( _PATH_CBCP_USERS, "r" );
	if ( userfile == NULL ){
	    dbglog( "Can't open callback user file: %s %m",
		    _PATH_CBCP_USERS );
	    dbglog( "Allow user definied callback." );
	    us->us_allowed = ( 1 << CB_CONF_USER );
	}else
	{
	    if ( fstat(fileno(userfile), &sbuf) < 0) {
		dbglog("Cannot stat userfile file %s: %m",
			_PATH_CBCP_USERS );
	    } else if ((sbuf.st_mode & (S_IRWXG | S_IRWXO)) != 0) {
		dbglog("Warning - user file %s has world and/or group access",
			_PATH_CBCP_USERS );
	    }

	    us->us_allowed = ( 1 << CB_CONF_NO ); /* Assume, no callback allowed */

	    if (getword(userfile, uname, &newline, _PATH_CBCP_USERS)){	/* file not empty */
		newline = 1;
		best_fit = 0;
		*option = 0;
		for (;;) {
		    /*
		     * Skip until we find a word at the start of a line.
		     */
		    while (!newline && getword(userfile, uname, 
				&newline, _PATH_CBCP_USERS))
			;
		    if (!newline)
			break;		/* got to end of file */

		    /*
		     * Got a user - check if it's a match or a wildcard.
		     */
		    got_fit = cbcp_check_user( peer_authname, uname );
		    if ( got_fit <= best_fit ){
			newline = 0;
			continue;
		    }

		    /* Read the options */
		    best_fit = got_fit;
		    if (getword(userfile, option, &newline, _PATH_CBCP_USERS))
			break;

		    if ( newline )
			*option = 0;

		    if ( best_fit == 100 )
			break;
		}
	    }

	    switch ( *option ){
		case '-' : us->us_allowed = ( 1 << CB_CONF_NO ); break;
		case '*' :
		case  0  : us->us_allowed = ( 1 << CB_CONF_USER ); break;
		default  : us->us_allowed = ( 1 << CB_CONF_ADMIN );
			   us->us_number = strdup( option ); break;
	    }
	    fclose( userfile );
	}    
    }
    else
	us->us_allowed = ( 1 << CB_CONF_USER );

    /* We always accept no callback users */	
    us->us_allowed |= ( 1 << CB_CONF_NO );
    
    if (cbcp_init_hook)
	cbcp_init_hook( us );

    cbcp_sendreq( us );
}    


/* make cbcp request packet & send it */
    static void
cbcp_sendreq (arg)
    void *arg;
{
    cbcp_state *us=(cbcp_state *)arg;
    u_char cb_allow = us->us_allowed;
    u_char buf[256];
    u_char *bufp = buf;
    int len = 0;

    us->us_count++;
    if (us->us_count<=CBCP_MAXRETRY)
	TIMEOUT( cbcp_sendreq, arg, CBCP_DEFTIMEOUT );
    else
    {
	lcp_close(0, "Sorry, CBCP not responding.");
	return;
    }
    dbglog("cbcp_sendreq cb_allowed=%d", cb_allow);


    if (cb_allow & ( 1 << CB_CONF_USER ) ) {
	dbglog("cbcp_sendreq CONF_USER");
	PUTCHAR(CB_CONF_USER, bufp);
	len+=3;
	PUTCHAR(3 , bufp);
	PUTCHAR(us->us_delay, bufp);
    }

    if (cb_allow & ( 1 << CB_CONF_ADMIN ) ) {
	dbglog("cbcp_sendreq CONF_ADMIN");
	PUTCHAR(CB_CONF_ADMIN, bufp);
	len += 3;
	PUTCHAR(3 , bufp);
	PUTCHAR(us->us_delay, bufp);
    }

    if (cb_allow & ( 1 << CB_CONF_NO ) ) {
	dbglog("cbcp_sendreq CONF_NO");
	PUTCHAR(CB_CONF_NO, bufp);
	len += 3;
	PUTCHAR(3 , bufp);
	PUTCHAR(us->us_delay, bufp);
    }

    if (len)
	cbcp_send(us, CBCP_REQ, buf, len);
    else
    {
	dbglog("cbcp: no available options to client!");    
    }	
}  

/* Received CBCP response, make ACK */
    static void
cbcp_recvresp (us, pckt, len)
    cbcp_state *us;
    char *pckt;
    int len;
{
    u_char type, addr_type;
    int opt_len;
    char address[256];

    if (len) {
	GETCHAR(type, pckt);
	GETCHAR(opt_len, pckt);

	if (!(( 1 << type )& us->us_allowed )) {
	    dbglog("CBCP received options not allowed on server!");
	    return;	    
	}

	if ((type!= CB_CONF_NO ) &&
		(type!= CB_CONF_USER ) &&
		(type!= CB_CONF_ADMIN )) {
	    dbglog("CBCP received BAD Response: too more or unknown options %d",type);
	    return;	    
	}

	UNTIMEOUT( cbcp_sendreq, us );
	us->us_count = 0;


	if (opt_len > 2)
	    GETCHAR(us->us_delay, pckt)
		if ( us->us_delay < 5 ) 
		    us->us_delay = 5;

	if (opt_len > 4) {
	    GETCHAR(addr_type, pckt); /*  Address Type mezo elvesztve  !!! */
	    memcpy(address, pckt, opt_len - 4);
	    address[opt_len - 4] = 0;
	    if (address[0])
		dbglog("peer will callback the client on: %s", address);
	    us->us_number=strdup( address );
	}

	us->us_type = ( 1 << type );    
	cbcp_sendack( us );
    }
    else
    {
	dbglog("CBCP received BAD Response: size to small");
    }
}

/* Send the CBCP_ACK packet */
    static void
cbcp_sendack (arg)
    void *arg;
{
    cbcp_state *us= (cbcp_state *)arg;
    u_char cb_type;
    u_char buf[256];
    u_char *bufp = buf;
    int len = 0;

    stop_iface = (cbcp_state *)arg;
    cb_type = us->us_type;

    dbglog("cbcp_sendack cb_type=%d", cb_type);

    us->us_count++;
    if (us->us_count<=CBCP_MAXRETRY)
	TIMEOUT( cbcp_sendack, arg, CBCP_DEFTIMEOUT );
    else
    {
	lcp_close(0, "Sorry, CBCP not responding.");
	return;
    }

#if 0
    if (!cb_type)
	lcp_down(us->us_unit);
#endif

    if (cb_type == (1 << CB_CONF_USER )) {
	dbglog("cbcp_sendack CONF_USER");
	PUTCHAR(CB_CONF_USER, bufp);
	len = 3 + 1 + strlen(us->us_number) + 1;
	PUTCHAR(len , bufp);
	PUTCHAR(us->us_delay, bufp); /* delay */
	PUTCHAR(1, bufp); /* Elvesztett byte... */
	BCOPY(us->us_number, bufp, strlen(us->us_number) + 1);
	cbcp_send(us, CBCP_ACK, buf, len);
	/*	lcp_close( 2, "Illegal, but required to server..." ); */
	callback_in_progress = us->us_unit + 1;
	return;
    }

    if (cb_type == (1 << CB_CONF_ADMIN )) {
	dbglog("cbcp_sendack CONF_ADMIN");
	PUTCHAR(CB_CONF_ADMIN, bufp);
	len = 3;
	PUTCHAR(len , bufp);
	PUTCHAR(us->us_delay, bufp); /* delay */
	PUTCHAR(0, bufp);
	cbcp_send(us, CBCP_ACK, buf, len);
	/*	lcp_close( 2, "Illegal, but required to server..." ); */
	callback_in_progress = us->us_unit + 1;
	return;
    }

    if (cb_type == (1 << CB_CONF_NO )) {
	dbglog("cbcp_sendack CONF_NO");
	PUTCHAR(CB_CONF_NO, bufp);
	len = 2;
	PUTCHAR(len , bufp);
	PUTCHAR(0, bufp);
	cbcp_send(us, CBCP_ACK, buf, len);
	if (us->us_count<=1)
	    network_phase(us->us_unit);
	return;
    }

    dbglog("CBCP - Bad options in Ack routine.");
}
 
/* CBCP coming succesful up */
void cbcp_stop()
{
    if ( stop_iface && lcp_allowoptions[stop_iface->us_unit].neg_cbcp )
    {
	UNTIMEOUT( cbcp_sendack, stop_iface );
	cbcp_start_callback( stop_iface );
    }
}

/* The server side coming up & client 'ack-ed' */
void cbcp_start_callback (us)
    cbcp_state *us;
{
    lcp_allowoptions[us->us_unit].neg_cbcp=0;

    dbglog("cbcp_start_callback running");
}

/* The client side coming up: server allowed the callback */
    static void
cbcp_up(us)
    cbcp_state *us;
{
    persist = 0;
    status = EXIT_CALLBACK;
    lcp_close(0, "Call me back, please");
    lcp_wantoptions[us->us_unit].neg_cbcp = 0;
    dbglog("cbcp_up called");
    lcp_close(us->us_unit, "Call me back, please");
}

/* The main module gets the script with parameters to run */
char *cbcp_get_script()
{
    cbcp_state *us = &cbcp[(callback_in_progress & CBCP_NCLIENT)-1];
    char script[ 256 ];

    if ( callback_in_progress & CBCP_CLIENT )
	sprintf( script, "%s %d", _PATH_CBCP_CLIENT, us->us_delay );
    else
	sprintf( script, "%s %d %s", _PATH_CBCP_SERVER, 
		us->us_delay, us->us_number );

    return strdup( script );        
}

/* give me the hit rate. wild cars '*?' valids */
int cbcp_check_user ( user, mask )
    char *user;
    char *mask;
{
    char *curr_user = user;
    char *curr_mask = mask;
    char *find, backp = 0;
    int  count, len = 0;
    
    if ( !strcasecmp( user, mask ))
	return 100;
	
    if ( !strcmp( mask, "*" ))
	return 1;
	
    if ( !*user )
	return 0;

    count = 0;

    while(( find = strpbrk( curr_mask, "*?" )) != 0 ) {
	if ( find != curr_mask ){
	    len = find - curr_mask;
	    if ( strncmp( curr_user, curr_mask, len ))
		break;	    
	}

	curr_mask += len + 1;
	curr_user += len;
	count += len;
	if ( *curr_user == 0 )
	    break;
	
	if ( *find == '?' ) {
	    curr_user++;
	    if ( *curr_user == 0 )
		break;
	} else {
	    if ( *curr_mask == 0 )
		break;
		
	    if ( ( find = strpbrk( curr_mask, "*?" )) != 0 ){
		backp = *find;
		*find = 0;
	    }
	    curr_user = strstr( curr_user, curr_mask );
	    if ( find )
		*find = backp;
	    if ( !curr_user )
		break;
	    
	    find = strpbrk( curr_mask, "*?" );
	    if ( find )
		len = find - curr_mask;
	    else
		len = strlen( curr_mask );
		
	    curr_mask += len;
	    curr_user += len;
	    count += len;
	}	
    }
    
    if ( *curr_user && *curr_mask && !strcmp( curr_user, curr_mask ))
	count += strlen( curr_user );

    return ( count * 100 / strlen( user ) );
}

